from langchain.llms.base import LLM
from langchain.callbacks.manager import CallbackManagerForLLMRun
from langchain.llms.utils import enforce_stop_tokens
from langchain.utils import get_from_dict_or_env
from pydantic.class_validators import root_validator

from typing import Any, Dict, List, Optional
from dwai.bailian.model_api.completions import Completions

class AliBlLLM(LLM):
    client: Any
    app_id: str
    app_key: str
    api_base: str
    history: Optional[List[Dict[str, Any]]] = []
    stream: bool = False
    model: str = "qwen-plus-v1"
    parameters: Optional[Dict[str, Any]]

    def __call__(self, *args, **kwargs):
        return super().__call__(*args, **kwargs)

    @property
    def _llm_type(self) -> str:
        return "DwBlLLM"

    @root_validator()
    def validate_environment(cls, values: Dict) -> Dict:

        if values["client"] is not None:
            return values

        values["app_id"] = get_from_dict_or_env(
            values,
            "app_id",
            "APP_ID",
        )

        values["app_key"] = get_from_dict_or_env(
            values,
            "app_key",
            "APP_KEY",
        )

        values["api_base"] = get_from_dict_or_env(
            values,
            "api_base",
            "API_BASE",
        )

        try:
            values["client"] = Completions(values["app_id"], values["app_key"], values["api_base"])
        except AttributeError:
            raise ValueError(
                "`openai` has no `ALI BL SDK` attribute, this is likely "
                "due to an old version of the openai package. Try upgrading it "
                "with `pip3.10 install --no-cache-dir  dwai==0.2.1`."
            )

        return values

    def _call(
            self,
            prompt: str,
            stop: Optional[List[str]] = None,
            run_manager: Optional[CallbackManagerForLLMRun] = None,
            **kwargs: Any,
    ) -> str:
        try:

            parameters = {
                "top_p": 0.0,
                "biz_params": None,
                "has_thoughts": False,
                "doc_reference_type": None
            } if not self.parameters else self.parameters

            parameters["model"] = self.model
            history = [] if not self.history else self.history

            response = self.client.run_v1(prompt, history, self.stream, **parameters)
            text = response.get("output", {}).get("text")
        except Exception as e:
            raise RuntimeError(f"Error raised by ali bl service: {e}")

        if stop is not None:
            text = enforce_stop_tokens(text, stop)
        return text
